
var g_Profiles = [];
var g_CreatedObjects = [];

function RemoveObjects()
{
	for ( var i = 0; i < g_CreatedObjects.length; ++i )
		moi.geometryDatabase.removeObject( g_CreatedObjects[i] );
		
	g_CreatedObjects = [];
}

function WrapWithObjectList( obj )
{
	var list = moi.geometryDatabase.createObjectList();
	list.addObject( obj );
	return list;
}

function ZScale( profile, origin, refA, refB )
{
	var factory = moi.command.createFactory( 'scale1d' );
	factory.setInput( 0, WrapWithObjectList(profile) );
	factory.setInput( 1, origin );
	factory.setInput( 3, refA );
	factory.setInput( 4, refB );
	
	var scaled_profile = factory.calculate().item(0);
	factory.cancel();
	
	return scaled_profile;
}

function UniformScale( obj, scale_factor )
{
	var factory = moi.command.createFactory( 'scale' );
	factory.setInput( 0, WrapWithObjectList(obj) );
	factory.setInput( 1, moi.vectorMath.createPoint(0,0,0) );
	factory.setInput( 2, scale_factor );
	
	var scaled_obj = factory.calculate().item(0);
	factory.cancel();
	
	return scaled_obj;
}

function Move( obj, basept, targetpt )
{
	var factory = moi.command.createFactory( 'move' );
	factory.setInput( 0, WrapWithObjectList(obj) );
	factory.setInput( 1, basept );
	factory.setInput( 2, targetpt );
	
	var moved_obj = factory.calculate().item(0);
	factory.cancel();
	
	return moved_obj;
}

function Revolve( profile, axis_begin, axis_end )
{
	var factory = moi.command.createFactory( 'revolve' );
	factory.setInput( 0, WrapWithObjectList(profile) );
	factory.setInput( 1, axis_begin );
	factory.setInput( 2, axis_end );
	
	var rev = factory.calculate().item(0);
	factory.cancel();
	
	return rev;
}

function BuildRing( diam, width, profile, xpos, ypos )
{
	var bounds = profile.getBoundingBox( true /* true = use high accuracy bounding box */ );
	
	var axis_begin = moi.vectorMath.createPoint( bounds.min.x - diam/2, bounds.min.y, bounds.min.z );
	var axis_end = moi.vectorMath.createPoint( axis_begin.x, axis_begin.y, bounds.max.z );
	var height_pt = moi.vectorMath.createPoint( axis_begin.x, axis_begin.y, axis_begin.z + width );
	
	var scaled_profile = ZScale( profile, axis_begin, axis_end, height_pt );
	
	var ring = Revolve( scaled_profile, axis_begin, axis_end );
	
	ring = Move( ring, axis_begin, moi.vectorMath.createPoint( xpos, ypos, 0 ) );
	
	g_CreatedObjects.push( ring );
}

function CreateObjects()
{
	// Get diameter, width, and profile name from UI.
	var diam = parseFloat( moi.ui.commandUI.size.value );
	var width = parseFloat( moi.ui.commandUI.width.value );
	var profile_name = moi.ui.commandUI.profile.value;

	// Get profile curve from label.	
	var profile;
	if ( profile_name == 'A' )
		profile = g_Profiles[0];
	else if ( profile_name == 'B' )
		profile = g_Profiles[1];
	else
		profile = g_Profiles[2];
		
	
	// Build the regular size ring and place it at x = 0, y = 25
	BuildRing( diam, width, profile, 0, 25 );
	
	// Build the size-1/4 ring and place it at x = -20, y = -20
	BuildRing( diam-0.2, width, profile, -20, -20 );
	
	// Build the size+1/4 ring and place it at x = 20, y = -20
	BuildRing( diam+0.2, width, profile, 20, -20 );
	

	// Scale all by 1.05 for shrinkage allowance.
	for ( var i = 0; i < g_CreatedObjects.length; ++i )
		g_CreatedObjects[i] = UniformScale( g_CreatedObjects[i], 1.05 );


	// Add objects to geometry database.	
	for ( var i = 0; i < g_CreatedObjects.length; ++i )
		moi.geometryDatabase.addObject( g_CreatedObjects[i] );

}

function SaveSTLFile()
{
	var filename = moi.filesystem.getSaveFileName( 'Save STL file', 'STL files (*.stl)|*.stl' );
	if ( filename == '' )
		return;
		
	moi.geometryDatabase.saveAs( filename, 'NoUI=true;Angle=3;MaxLength=1;MaxLengthApplyTo=all' );
}

function Update()
{
	RemoveObjects();
	CreateObjects();
}

function FindProfiles()
{
	// Find profiles named A, B, C.

	var ProfileA = false, ProfileB = false, ProfileC = false;
	
	var objs = moi.geometryDatabase.getObjects();
	for ( var i = 0; i < objs.length; ++i )
	{
		var obj = objs.item(i);
		if ( obj.name == 'A' )
			ProfileA = obj;
		else if ( obj.name == 'B' )
			ProfileB = obj;
		else if ( obj.name == 'C' )
			ProfileC = obj;	
	}
	
	if ( !ProfileA || !ProfileB || !ProfileC )
	{
		moi.ui.alert( 'Profile curves not found, please load 3DM file with profiles before running this command.' );
		return false;
	}
	
	g_Profiles[0] = ProfileA;
	g_Profiles[1] = ProfileB;
	g_Profiles[2] = ProfileC;
	return true;
}

function DoMakeRing()
{
	if ( !FindProfiles() )
		return;

	Update();

	var commandDialog = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !commandDialog.waitForEvent() )
		{
			// Command is being canceled.
			RemoveObjects();
			return;
		}
			
		if ( commandDialog.event == 'done' )
			break;
			
		// If size, width, or profile have been changed update the model.
		if ( commandDialog.event == 'size' || commandDialog.event == 'width' || commandDialog.event == 'profile' )
			Update();
	}
	
	SaveSTLFile();
	
	RemoveObjects();
}

DoMakeRing();
